searchNodes=[{"doc":"Reusing the description from the paper:Pattern synonyms allow to abstract over patterns used in pattern matching, notably by allowing to use computation instead of being limited to concrete data.Pattern metonyms are an implementations of this, but because of the limitation of the language, it can not be considered the same, so metonyms was chosen as a synonym to synonyms.Unlike in Haskell, few metonym definitions can be used with case/2, other forms of patterns can only be used in combination with view/2.","ref":"PatternMetonyms.html","title":"PatternMetonyms","type":"module"},{"doc":"Macro used to define pattern metonyms.There are three types of pattern, listed below with examples:Implicitly BidirectionalThe simplest type of pattern, because of its symmetry requirement, it can only be defined using concrete data (or adapted macro). Therefore no computation is allowed, and they are thus compatible with case and function heads. They take the form:pattern &lt;name&gt;(&lt;[variables]&gt;) = &lt;pattern&gt;where pattern reuses the variablesiex&gt; defmodule DoctestTPX do ...&gt; import PatternMetonyms ...&gt; ...&gt; pattern ok(x) = {:ok, x} ...&gt; pattern error(x) = {:error, x} ...&gt; ...&gt; pattern cons(x, xs) = [x | xs] ...&gt; ...&gt; def foo(x) do ...&gt; view x do ...&gt; ok(a) -&gt; a ...&gt; error(b) -&gt; b ...&gt; end ...&gt; end ...&gt; ...&gt; def bar(x) do ...&gt; case x do ...&gt; ok(a) -&gt; a ...&gt; error(b) -&gt; b ...&gt; end ...&gt; end ...&gt; ...&gt; def baz(ok(a) ), do: a ...&gt; def baz(error(b)), do: b ...&gt; ...&gt; def mk_ok(x), do: ok(x) ...&gt; ...&gt; def blorg(xs) do ...&gt; view xs do ...&gt; cons(x, xs) -&gt; cons(x, Enum.map(xs, fn x -&gt; -x end)) ...&gt; end ...&gt; end ...&gt; end iex&gt; DoctestTPX.foo({:ok, :banana}) :banana iex&gt; DoctestTPX.foo({:error, :split}) :split iex&gt; DoctestTPX.bar({:ok, :peach}) :peach iex&gt; DoctestTPX.baz({:error, :melba}) :melba iex&gt; DoctestTPX.mk_ok(:melba) {:ok, :melba} iex&gt; DoctestTPX.blorg([1, 2, 3]) [1, -2, -3]UnidirectionalThis type of pattern is read only, it may be used as abstraction over pattern matching on concrete data type that can not be reused to construct data, or as abstraction over views, as explained in view/2. They take the form:pattern &lt;name&gt;(&lt;[variables]&gt;) &lt;- &lt;pattern&gt; pattern &lt;name&gt;(&lt;[variables]&gt;) &lt;- (&lt;function&gt; -&gt; &lt;pattern&gt;)where pattern reuses the variables. (function -&gt; pattern) is called a viewiex&gt; defmodule DoctestTPY do ...&gt; import PatternMetonyms ...&gt; ...&gt; pattern head(x) &lt;- [x | _] ...&gt; ...&gt; pattern rev_head(x) &lt;- (reverse -&gt; head(x)) ...&gt; ...&gt; def reverse(xs), do: Enum.reverse(xs) ...&gt; ...&gt; def foo(xs) do ...&gt; view xs do ...&gt; head(x) -&gt; x ...&gt; [] -&gt; [] ...&gt; end ...&gt; end ...&gt; ...&gt; def bar(x) do ...&gt; case x do ...&gt; head(a) -&gt; a ...&gt; [] -&gt; [] ...&gt; end ...&gt; end ...&gt; ...&gt; def baz(head(a)), do: a ...&gt; ...&gt; def blorg(xs) do ...&gt; view xs do ...&gt; rev_head(x) -&gt; x ...&gt; end ...&gt; end ...&gt; end iex&gt; DoctestTPY.foo([1, 2, 3]) 1 iex&gt; DoctestTPY.bar([1, 2, 3]) 1 iex&gt; DoctestTPY.baz([1, 2, 3]) 1 iex&gt; DoctestTPY.blorg([1, 2, 3]) 3Explicitly bidirectionalThis type of pattern allows the same kind of abstraction as unidirectional one, but also permit defining how to construct data from computation (if necessary). They take the form:pattern (&lt;name&gt;(&lt;[variables]&gt;) &lt;- (&lt;function&gt; -&gt; &lt;pattern&gt;)) when &lt;name&gt;(&lt;[variables]&gt;) = &lt;builder&gt;where pattern and builder reuse the variables. (function -&gt; pattern) is called a viewiex&gt; defmodule DoctestTPZ do ...&gt; import PatternMetonyms ...&gt; ...&gt; pattern (snoc(x, xs) &lt;- (unsnoc -&gt; {x, xs})) ...&gt; when snoc(x, xs) = Enum.reverse([x | Enum.reverse(xs)]) ...&gt; ...&gt; defp unsnoc([]), do: :error ...&gt; defp unsnoc(xs) do ...&gt; [x | rev_tail] = Enum.reverse(xs) ...&gt; {x, Enum.reverse(rev_tail)} ...&gt; end ...&gt; ...&gt; def foo(xs) do ...&gt; view xs do ...&gt; snoc(x, _) -&gt; x ...&gt; [] -&gt; [] ...&gt; end ...&gt; end ...&gt; ...&gt; def bar(xs) do ...&gt; view xs do ...&gt; snoc(x, xs) -&gt; snoc(-x, xs) ...&gt; [] -&gt; [] ...&gt; end ...&gt; end ...&gt; end iex&gt; DoctestTPZ.foo([1, 2, 3]) 3 iex&gt; DoctestTPZ.bar([1, 2, 3]) [1, 2, -3]Patterns using a view can not be used with case.Remote function can be used within a view, but the __MODULE__ alias won't work because the expansion is not done at the usage site. It is not yet determined which behavior is desired.iex&gt; defmodule DoctestTPA do ...&gt; import PatternMetonyms ...&gt; ...&gt; pattern rev_head(x) &lt;- (Enum.reverse -&gt; [x | _]) ...&gt; ...&gt; def blorg(xs) do ...&gt; view xs do ...&gt; rev_head(x) -&gt; x ...&gt; end ...&gt; end ...&gt; end iex&gt; DoctestTPA.blorg([1, 2, 3]) 3Unknown yet if anonymous functions can be supported.Guards within a pattern definition is considered undefined behavior, it may work, but it depends on the context. Consider that if the behavior gets a specification, it would be the removal of the possibility of using them. Patterns using a view pattern are the recommend approach. For example:pattern heart(n) &lt;- (less_than_3 -&gt; {:ok, n})","ref":"PatternMetonyms.html#pattern/1","title":"PatternMetonyms.pattern/1","type":"macro"},{"doc":"Macro substitute for case/2 capable of using pattern metonyms.Custom case able to use pattern metonyms defined with this module. Largely unoptimized, try to avoid side effect in your pattern definitions as using them multiple time in view will repeat them, but might not later on.View pattern ((function -&gt; pattern)) may be used raw in here.View patterns are simply a pair of a function associated with a pattern where the function will be applied to the data passed to view and the result will be matched with the pattern.iex&gt; import PatternMetonyms iex&gt; view self() do ...&gt; (is_pid -&gt; true) -&gt; :ok ...&gt; _ -&gt; :ko ...&gt; end :okGuards can be used outside of the view pattern or the pattern metonym.iex&gt; import PatternMetonyms iex&gt; view -3 - :rand.uniform(2) do ...&gt; (abs -&gt; x) when x &gt; 3 -&gt; :ok ...&gt; (abs -&gt; x) when x &lt; 3 -&gt; :ko ...&gt; x -&gt; x ...&gt; end :okRemote calls can be used directly within a view pattern.iex&gt; import PatternMetonyms iex&gt; view :banana do ...&gt; (Atom.to_string -&gt; &quot;ba&quot; &lt;&gt; _) -&gt; :ok ...&gt; _ -&gt; :ko ...&gt; end :okAnonymous functions can be used within a view pattern. They can be either used as stored within a variable:iex&gt; import PatternMetonyms iex&gt; fun = &amp;inspect(&amp;1, pretty: &amp;2) iex&gt; view :banana do ...&gt; (fun.(true) -&gt; str) -&gt; str ...&gt; end &quot;:banana&quot;Or defined directly using Kernel.SpecialForms.fn/1 only:iex&gt; import PatternMetonyms iex&gt; view 3 do ...&gt; (fn x -&gt; x + 2 end -&gt; n) -&gt; n + 1 ...&gt; end 6","ref":"PatternMetonyms.html#view/2","title":"PatternMetonyms.view/2","type":"macro"}]